class GraphMatrix

  def initialize(args)
    @type = args[:type]
    @weight_limit = args[:weight_limit]
    if args.has_key?(:num_vertices)
      @edges = Array.new(args[:num_vertices]) { Array.new(args[:num_vertices]) }
      initialize_helper(args[:num_vertices], args[:num_edges])
    else
      @edges = args[:edge_matrix]
    end
  end

  def get_out_edges(vertex)
    return if (vertex < 0 || vertex >= @edges.length)
    vertices = []
    (0..(@edges.length - 1)).each do |index|
      vertices << index if @edges[vertex][index] != nil
    end
    vertices
  end

  def get_in_edges(vertex)
    return if (vertex < 0 || vertex >= @edges.length)
    vertices = []
    (0..(@edges.length - 1)).each do |index|
      vertices << index if @edges[index][vertex] != nil
    end
    vertices
  end

  def get_edge_weight(from, to)
    @edges[from][to]
  end

  def vertices
    vertices = []
    (0..(@edges.length - 1)).each do |index|
      vertices << index
    end
    vertices
  end

  def edges
    result = []
    (0..(@edges.length - 1)).each do |index|
      (0..(@edges.length - 1)).each do |s_index|
        next if @type == 'undirected' && s_index < index
        result << [index, s_index] if @edges[index][s_index] != nil
      end
    end
    result
  end

  def edges_with_weights
    result = []
    (0..(@edges.length - 1)).each do |index|
      (0..(@edges.length - 1)).each do |s_index|
        next if @type == 'undirected' && s_index < index
        result << [index, s_index, @edges[index][s_index]] if @edges[index][s_index] != nil
      end
    end
    result
  end

  private

  def initialize_helper(num_vertices, num_edges)
    counter = 0
    while counter < num_edges
      from = rand(num_vertices)
      to = rand(num_vertices)
      next if from == to
      counter = add_edge_helper(to, from, counter)
    end
  end

  def add_edge_helper(to, from, counter)
    return counter if @edges[from][to] != nil
    @edges[from][to] = generate_edge_weight
    @edges[to][from] = @edges[from][to] if @type == 'undirected'
    counter + 1
  end

  def generate_edge_weight
    if @weight_limit == 0
      1
    else
      rand(@weight_limit)
    end
  end


  attr_writer :edges, :type, :weight_limit
end